@@ -1,8 +1,10 @@
 import os
 import time
 import subprocess
+import tempfile
 
 from worker import BupWorker
+from sudo import sudo
 
 def noop(*args):
 	pass
@@ -11,9 +13,8 @@ class BupManager:
 	def __init__(self, cfg):
 		self.config = cfg
 
-		dirname = os.path.realpath(os.path.dirname(__file__))
-		self.mountPath = os.path.join(dirname, "mnt")
-		self.fuseMountPath = os.path.join(dirname, "mnt-bup")
+		self.mountPath = tempfile.mkdtemp(prefix="bups-mnt-")
+		self.fuseMountPath = tempfile.mkdtemp(prefix="bups-fuse-")
 
 		self.bupPath = self.mountPath
 		if "path" in cfg["mount"] and cfg["mount"]["path"]:
@@ -38,8 +39,10 @@ class BupManager:
 		ctx = {}
 
 		def backupDir(dir_data):
-			dirpath = dir_data["path"].encode('ascii')
-			backupName = dir_data["name"].encode('ascii')
+			dirpath = dir_data["path"].encode("ascii")
+			backupName = dir_data["name"].encode("ascii")
+			excludePaths = [x.encode("ascii") for x in dir_data.get("exclude", [])]
+			excludeRxs = [x.encode("ascii") for x in dir_data.get("excluderx", [])]
 
 			ctx = {
 				"path": dirpath,
@@ -51,7 +54,10 @@ class BupManager:
 
 			callbacks["onstatus"]("Backing up "+backupName+": indexing files...", ctx)
 
-			self.bup.index(dirpath, {}, {
+			self.bup.index(dirpath, {
+				"exclude_paths": excludePaths,
+				"exclude_rxs": excludeRxs
+			}, {
 				"onread": lambda line: None,
 				"onprogress": onprogress
 			})
@@ -141,21 +147,14 @@ class BupManager:
 
 		time.sleep(1)
 
+		os.rmdir(self.fuseMountPath)
+
 		callbacks["onstatus"]("Unmounting filesystem...")
 		self.bupUnmount(callbacks)
 
 		self.mounted = False
 		callbacks["onfinish"]({})
 
-	def get_sudo(self, cmd):
-		if os.geteuid() != 0:
-			if "DISPLAY" in os.environ:
-				sudo = "gksu"
-			else:
-				sudo = "sudo sh -c"
-			cmd = sudo+" \""+cmd+"\""
-		return cmd
-
 	def bupMount(self, callbacks={}):
 		if not "onerror" in callbacks:
 			callbacks["onerror"] = noop
@@ -171,7 +170,7 @@ class BupManager:
 			callbacks["onerror"]("WARN: filesystem already mounted", {})
 		else:
 			cmd = "mount -t "+cfg["mount"]["type"]+" "+cfg["mount"]["target"]+" "+self.mountPath+" -o "+cfg["mount"]["options"]
-			res = subprocess.call([self.get_sudo(cmd)], shell=True)
+			res = sudo(cmd)
 			if res == 32:
 				callbacks["onerror"]("WARN: filesystem busy", {})
 			elif res != 0:
@@ -188,10 +187,13 @@ class BupManager:
 
 		cfg = self.config
 		if cfg["mount"]["type"] == "": # Nothing to unmount
+			os.rmdir(self.mountPath)
 			return True
 
 		cmd = "umount "+self.mountPath
-		res = subprocess.call([self.get_sudo(cmd)], shell=True)
+		res = sudo(cmd)
 		if res != 0:
 			callbacks["onerror"]("WARN: could not unmount samba filesystem ["+str(res)+"]", {})
-			return False
\ No newline at end of file
+			return False
+
+		os.rmdir(self.mountPath)
\ No newline at end of file
